from datetime import date
from json import dumps

from django import forms
from django.contrib.auth import get_user_model
from django.db.models import Q
from django.db.utils import IntegrityError
from django.utils.crypto import get_random_string
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _
from verified_email_field.forms import VerifiedEmailField

from ..models.agegroup import AgeGroup
from ..models.courses import Course, CourseRegistration
from ..models.department import Department
from ..models.events import Event, EventRegistration
from ..models.fields import DAY_OF_WEEK
from ..models.place import Place
from ..models.roles import Leader, Parent, Participant
from ..models.subjects import Subject, SubjectGroup, SubjectType
from ..utils import get_age, get_birth_date
from .form import FormMixin
from .widgets import RadioSelectBootstrap

User = get_user_model()


class SubjectFilterForm(FormMixin, forms.Form):
    q               = forms.CharField(label=_('Search term'), required=False)
    course_types    = forms.ModelMultipleChoiceField(queryset=None, label=_('Course type'), required=False)
    event_types     = forms.ModelMultipleChoiceField(queryset=None, label=_('Event type'), required=False)
    departments     = forms.ModelMultipleChoiceField(queryset=None, label=_('Department'), required=False)
    groups          = forms.ModelMultipleChoiceField(queryset=None, label=_('Group'), required=False)
    leaders         = forms.ModelMultipleChoiceField(queryset=None, label=_('Leader'), required=False)
    places          = forms.ModelMultipleChoiceField(queryset=None, label=_('Place'), required=False)
    age_groups      = forms.ModelMultipleChoiceField(queryset=None, label=_('Age group'), required=False)
    days_of_week    = forms.MultipleChoiceField(label=_('Day of week'),
                                                choices=tuple(sorted(DAY_OF_WEEK.items())), required=False)
    past            = forms.BooleanField(label=_('Include past subjects'), required=False)
    reg_active      = forms.BooleanField(label=_('Available for registration'), required=False)
    invisible       = forms.BooleanField(label=_('Show invisible'), required=False)

    _models = {
        SubjectType.COURSE: Course,
        SubjectType.EVENT:  Event,
    }

    def __init__(self, subject_type_type, subject_types, school_year, is_staff, data, **kwargs):
        super(SubjectFilterForm, self).__init__(data=data, **kwargs)
        self.subject_type_type = subject_type_type

        # pre filter subjects by initial params
        qs = self._models[subject_type_type].objects.filter(school_year=school_year)
        if len(subject_types) == 1:
            qs = qs.filter(subject_type=subject_types[0])
        else:
            qs = qs.filter(subject_type__in=subject_types)
        if not is_staff or 'invisible' not in data:
            qs = qs.filter(public=True)
        self.qs = qs

        subject_ids = tuple(qs.order_by('id').values_list('id', flat=True).distinct())
        if len(subject_types) == 1:
            del self.fields['course_types']
            del self.fields['event_types']
        elif subject_type_type == SubjectType.COURSE:
            del self.fields['event_types']
            self.fields['course_types'].queryset = SubjectType.objects.filter(id__in=(st.id for st in subject_types))
        elif subject_type_type == SubjectType.EVENT:
            del self.fields['course_types']
            self.fields['event_types'].queryset = SubjectType.objects.filter(id__in=(st.id for st in subject_types))

        self.fields['departments'].queryset  = Department.objects.filter(subjects__id__in=subject_ids).distinct()
        if self.fields['departments'].queryset.count() == 0:
            del self.fields['departments']

        self.fields['groups'].queryset  = SubjectGroup.objects.filter(subject_types__in=subject_types,
                                                                      subjects__id__in=subject_ids).distinct()
        if self.fields['groups'].queryset.count() == 0:
            del self.fields['groups']

        self.fields['leaders'].queryset = (Leader.objects.filter(subjects__id__in=subject_ids).distinct()
                                           .order_by('user__first_name', 'user__last_name'))
        if self.fields['leaders'].queryset.count() == 0:
            del self.fields['leaders']

        self.fields['places'].queryset  = Place.objects.filter(subjects__id__in=subject_ids).distinct()
        if self.fields['places'].queryset.count() == 0:
            del self.fields['places']

        self.fields['age_groups'].queryset = AgeGroup.objects.filter(subjects__id__in=subject_ids).distinct()
        if self.fields['age_groups'].queryset.count() <= 1:
            del self.fields['age_groups']

        if subject_type_type != SubjectType.COURSE:
            del self.fields['days_of_week']
        if subject_type_type != SubjectType.EVENT:
            del self.fields['past']
        if not is_staff:
            del self.fields['invisible']

        for f in self.fields:
            self.fields[f].help_text = None

    def get_queryset(self):
        if not self.is_valid():
            return self.qs
        qs = self.qs
        for word in self.cleaned_data['q'].split():
            qs = qs.filter(
                Q(name__icontains = word) |
                Q(description__icontains = word)
            )
        if self.cleaned_data.get('course_types'):
            qs = qs.filter(subject_type__in = self.cleaned_data['course_types'])
        elif self.cleaned_data.get('event_types'):
            qs = qs.filter(subject_type__in = self.cleaned_data['event_types'])
        if self.cleaned_data.get('departments'):
            qs = qs.filter(department__in = self.cleaned_data['departments'])
        if self.cleaned_data.get('groups'):
            qs = qs.filter(groups__in = self.cleaned_data['groups'])
        if self.cleaned_data.get('places'):
            qs = qs.filter(place__in = self.cleaned_data['places'])
        if self.cleaned_data.get('leaders'):
            qs = qs.filter(leaders__in = self.cleaned_data['leaders'])
        if self.cleaned_data.get('age_groups'):
            qs = qs.filter(age_groups__in = self.cleaned_data['age_groups'])
        if self.cleaned_data.get('days_of_week'):
            qs = qs.filter(times__day_of_week__in = self.cleaned_data['days_of_week'])
        if self.subject_type_type == SubjectType.EVENT and not self.cleaned_data['past']:
            qs = qs.filter(end_date__gte = now())
        if self.cleaned_data['reg_active']:
            qs = qs.filter(reg_from__lte=now()).exclude(reg_to__lte=now()).exclude(price=None)
        return qs.distinct()



class SubjectForm(FormMixin, forms.ModelForm):

    class Meta:
        model = Subject
        fields = ['description', 'risks', 'plan', 'evaluation']



class RegistrationForm(FormMixin, forms.ModelForm):

    def __init__(self, subject, user, **kwargs):
        super(RegistrationForm, self).__init__(**kwargs)
        self.user = user
        self.instance.subject = subject
        self.participants = user.leprikon_participants.exclude(
            birth_num__in=self.instance.subject.active_registrations.values_list('participant_birth_num', flat=True)
        ) if user.is_authenticated() else []
        self.parents = user.leprikon_parents.all() if user.is_authenticated() else []

        # choices for subject_variant
        if self.instance.subject.all_variants:
            self.fields['subject_variant'].widget.choices.queryset = self.instance.subject.variants
        else:
            del self.fields['subject_variant']

        self.fields['participant_age_group'].widget.choices.queryset = self.instance.subject.age_groups

        # dynamically required fields
        try:
            if get_age(get_birth_date(kwargs['data']['participant_birth_num'])) < 18:
                self.fields['has_parent1'].required = True
            else:
                self.fields['participant_phone'].required = True
                self.fields['participant_email'].required = True
        except:
            pass

        try:
            has_parent = ['has_parent1' in kwargs['data'], 'has_parent2' in kwargs['data']]
        except:
            has_parent = [False, False]
        for n in range(2):
            if has_parent[n]:
                for field in ['first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email']:
                    self.fields['parent{}_{}'.format(n + 1, field)].required = True

        # sub forms

        del kwargs['instance']

        if not user.is_authenticated():
            kwargs['prefix'] = 'email'
            self.email_form = self.EmailForm(**kwargs)

        class ParticipantSelectForm(FormMixin, forms.Form):
            participant = forms.ChoiceField(
                label   = _('Choose'),
                choices = [('new', _('new participant'))] + list((p.id, p) for p in self.participants),
                initial = 'new',
                widget  = RadioSelectBootstrap(),
            )
        kwargs['prefix'] = 'participant_select'
        self.participant_select_form = ParticipantSelectForm(**kwargs)

        class ParentSelectForm(FormMixin, forms.Form):
            parent = forms.ChoiceField(
                label   = _('Choose'),
                choices = [('new', _('new parent'))] + list((p.id, p) for p in self.parents),
                initial = 'new',
                widget  = RadioSelectBootstrap(),
            )
        kwargs['prefix'] = 'parent1_select'
        self.parent1_select_form = ParentSelectForm(**kwargs)
        kwargs['prefix'] = 'parent2_select'
        self.parent2_select_form = ParentSelectForm(**kwargs)

        kwargs['prefix'] = 'questions'
        self.questions_form = type(str('QuestionsForm'), (FormMixin, forms.Form), dict(
            (q.name, q.get_field()) for q in self.instance.subject.all_questions
        ))(**kwargs)

        self.agreement_forms = []
        for agreement in self.instance.subject.all_registration_agreements:
            kwargs['prefix'] = 'agreement_%s' % agreement.id
            form_attributes = {
                'agreement': agreement,
                'options': {},
            }
            for option in agreement.all_options:
                option_id = 'option_%s' % option.id
                form_attributes[option_id] = forms.BooleanField(
                    label=option.option,
                    required=option.required,
                )
                form_attributes['options'][option_id] = option
            AgreementForm = type(str('AgreementForm'), (FormMixin, forms.Form), form_attributes)
            self.agreement_forms.append(AgreementForm(**kwargs))

        kwargs['prefix'] = 'old_agreement'
        self.old_agreement_form = self.OldAgreementForm(**kwargs)

    def is_valid(self):
        return (
            (self.user.is_authenticated or self.email_form.is_valid()) and
            self.participant_select_form.is_valid() and
            self.questions_form.is_valid() and
            self.parent1_select_form.is_valid() and
            self.parent2_select_form.is_valid() and
            (not self.instance.old_registration_agreement or self.old_agreement_form.is_valid()) and
            all(agreement_form.is_valid() for agreement_form in self.agreement_forms) and
            super(RegistrationForm, self).is_valid()
        )

    def save(self, commit=True):
        # set user
        if self.user.is_authenticated():
            self.instance.user = self.user
        else:
            user = User.objects.filter(
                email=self.email_form.cleaned_data['email'].lower(),
            ).first() or User(
                email=self.email_form.cleaned_data['email'].lower(),
            )
            while not user.pk:
                user.username = get_random_string()
                user.set_password(get_random_string())
                try:
                    user.save()
                except IntegrityError:
                    # on duplicit username try again
                    pass
            self.instance.user = user

        # set price
        self.instance.price = (
            self.instance.subject_variant.price
            if self.instance.subject_variant
            else self.instance.subject.price
        )
        self.instance.answers = dumps(self.questions_form.cleaned_data)
        super(RegistrationForm, self).save(commit)

        # send mail
        try:
            self.instance.send_mail()
        except:
            import traceback
            from raven.contrib.django.models import client
            traceback.print_exc()
            client.captureException()

        # save / update participant
        if self.participant_select_form.cleaned_data['participant'] == 'new':
            try:
                participant = Participant.objects.get(
                    user        = self.instance.user,
                    birth_num   = self.instance.participant.birth_num,
                )
            except Participant.DoesNotExist:
                participant = Participant()
                participant.user = self.instance.user
        else:
            participant = Participant.objects.get(id=self.participant_select_form.cleaned_data['participant'])
        for attr in ['first_name', 'last_name', 'birth_num', 'age_group', 'street', 'city', 'postal_code',
                     'citizenship', 'phone', 'email', 'school', 'school_other', 'school_class', 'health']:
            setattr(participant, attr, getattr(self.instance.participant, attr))
        participant.save()

        # save / update first parent
        if self.instance.parent1:
            if self.parent1_select_form.cleaned_data['parent'] == 'new':
                parent = Parent()
                parent.user = self.instance.user
            else:
                parent = Parent.objects.get(id=self.parent1_select_form.cleaned_data['parent'])
            for attr in ['first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email']:
                setattr(parent, attr, getattr(self.instance.parent1, attr))
            parent.save()

        # save / update second parent
        if self.instance.parent2:
            if self.parent2_select_form.cleaned_data['parent'] == 'new':
                parent = Parent()
                parent.user = self.instance.user
            else:
                parent = Parent.objects.get(id=self.parent2_select_form.cleaned_data['parent'])
            for attr in ['first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email']:
                setattr(parent, attr, getattr(self.instance.parent2, attr))
            parent.save()

        for agreement_form in self.agreement_forms:
            for option_id, checked in agreement_form.cleaned_data.items():
                if checked:
                    self.instance.agreement_options.add(agreement_form.options[option_id])

    class OldAgreementForm(FormMixin, forms.Form):
        agreement = forms.BooleanField(label = _('Terms and Conditions agreement'))

    class EmailForm(FormMixin, forms.Form):
        email = VerifiedEmailField(label=_('Your email'), fieldsetup_id='RegistrationEmailForm')


class CourseRegistrationForm(RegistrationForm):

    class Meta:
        model = CourseRegistration
        exclude = ('subject', 'user', 'cancel_request', 'canceled')



class EventRegistrationForm(RegistrationForm):

    class Meta:
        model = EventRegistration
        exclude = ('subject', 'user', 'cancel_request', 'canceled')



class RegistrationAdminForm(forms.ModelForm):

    def __init__(self, data=None, *args, **kwargs):
        super(RegistrationAdminForm, self).__init__(data, *args, **kwargs)
        instance = kwargs.get('instance')
        initial = kwargs.get('initial')
        try:
            subject = Subject.objects.get(id=int(data['subject']))
        except (Subject.DoesNotExist, TypeError, KeyError, ValueError):
            subject = instance.subject if instance else Subject.objects.get(id=initial['subject'])
        self.fields['subject_variant'].widget.choices.queryset = subject.variants
        if not subject.variants.exists():
            self.fields['subject_variant'].required = False
        self.fields['participant_age_group'].widget.choices.queryset = subject.age_groups

        try:
            age = get_age(
                get_birth_date(data['participant_birth_num']),
                instance.created.date() if instance else date.today(),
            )
        except:
            age = get_age(
                get_birth_date(instance.participant.birth_num),
                instance.created.date(),
            ) if instance else 18
        if age < 18:
            self.fields['has_parent1'].required = True
        else:
            self.fields['participant_phone'].required = True
            self.fields['participant_email'].required = True

        try:
            has_parent = ['has_parent1' in data, 'has_parent2' in data]
        except:
            has_parent = [instance.has_parent1, instance.has_parent2] if instance else [False, False]
        for n in range(2):
            if has_parent[n]:
                for field in ['first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email']:
                    self.fields['parent{}_{}'.format(n + 1, field)].required = True

        # choices for subject_variant
        self.fields['agreement_options'].widget.choices = tuple(
            (agreement.name, tuple((option.id, option.name) for option in agreement.all_options))
            for agreement in subject.all_registration_agreements
        )



import re

from django import forms
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.db import models
from django.utils.translation import ugettext_lazy as _
from localflavor.cz.forms import CZPostalCodeField

from ..conf import settings
from ..utils import get_birth_date


class ColorInput(forms.TextInput):
    input_type = 'color'


class ColorField(models.CharField):
    default_validators = [RegexValidator(
        re.compile('^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$'),
        _('Enter a valid hex color.'),
        'invalid',
    )]

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 10
        super(ColorField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs['widget'] = ColorInput
        return super(ColorField, self).formfield(**kwargs)


class EmailField(models.EmailField):
    def to_python(self, value):
        v = super(EmailField, self).to_python(value)
        return v and v.lower()


class PriceField(models.DecimalField):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('decimal_places', settings.PRICE_DECIMAL_PLACES)
        kwargs.setdefault('max_digits',     settings.PRICE_MAX_DIGITS)
        super(PriceField, self).__init__(*args, **kwargs)



DAY_OF_WEEK = {
    1: _('Monday'),
    2: _('Tuesday'),
    3: _('Wednesday'),
    4: _('Thursday'),
    5: _('Friday'),
    6: _('Saturday'),
    7: _('Sunday'),
}


class DayOfWeekField(models.IntegerField):
    def __init__(self, *args, **kwargs):
        defaults = {
            'choices': tuple(sorted(DAY_OF_WEEK.items())),
        }
        defaults.update(kwargs)
        super(DayOfWeekField, self).__init__(*args, **defaults)



birth_num_regex = re.compile('^[0-9]{2}([0257][1-9]|[1368][0-2])[0-3][0-9]/?[0-9]{3,4}$')


def validate_birth_num(value):
    try:
        assert birth_num_regex.match(value) is not None
        get_birth_date(value)
        value = value.replace('/', '')
        if len(value) > 9:
            assert int(value) % 11 == 0
    except:
        raise ValidationError(
            message=_('Enter a valid birth number.'),
            code='invalid',
        )


class BirthNumberField(models.CharField):
    default_validators = [validate_birth_num]

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 11
        super(BirthNumberField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(BirthNumberField, self).deconstruct()
        del kwargs['max_length']
        return name, path, args, kwargs

    def clean(self, value, model_instance):
        value = super(BirthNumberField, self).clean(value, model_instance)
        return value if value[6] == '/' else '{}/{}'.format(value[:6], value[6:])



class _PostalCodeField(CZPostalCodeField, forms.CharField):
    '''
    CZPostalCodeField derived from CharField instead of just Field
    to support max_length
    '''


class PostalCodeField(models.CharField):
    def __init__(self, *args, **kwargs):
        defaults = {'max_length': 6}
        defaults.update(kwargs)
        super(PostalCodeField, self).__init__(*args, **defaults)

    def deconstruct(self):
        name, path, args, kwargs = super(PostalCodeField, self).deconstruct()
        del kwargs["max_length"]
        return name, path, args, kwargs

    def formfield(self, **kwargs):
        defaults = {'form_class': _PostalCodeField}
        defaults.update(kwargs)
        return super(PostalCodeField, self).formfield(**defaults)

    def clean(self, value, model_instance):
        return super(PostalCodeField, self).clean(
            value[3] == ' ' and value or '{} {}'.format(value[:3], value[3:]),
            model_instance,
        )



from urllib.parse import urlencode

from django import template

from .. import payment_url as _payment_url
from ..models import Gateway

register = template.Library()


@register.simple_tag(takes_context=True)
def payment_url(context, gateway, order_id, amount, currency=None, email=None, lang=None):
    return _payment_url(
        gateway=gateway,
        order_id=oreder_id,
        amount=amount,
        currency=currency,
        email=email or getattr(getattr(context.get('request'), 'user', None), 'email', ''),
        lang=lang,
    )



from django.db import models
from django.forms.widgets import PasswordInput
from django.utils.translation import ugettext_lazy as _


class PasswordField(models.CharField):
    def formfield(self, **kwargs):
        kwargs.update({'widget': PasswordInput})
        return super().formfield(**kwargs)


class Gateway(models.Model):
    CURRENCIES = ('CZK', 'EUR', 'USD')
    CURRENCY_BASE_UNITS = {
        'CZK': 100,
        'EUR': 100,
        'USD': 100,
    }
    LANGS = ('CS-CZ', 'SK-SK', 'EN-US', 'RU-RU', 'JA-JP')

    name = models.CharField(_('name'), max_length=64)
    slug = models.SlugField(_('slug'), max_length=64)
    merchant_id = models.CharField(_('merchant identifier'), max_length=64)
    shop_id = models.CharField(_('shop identifier'), max_length=64)
    secret_key = PasswordField(_('secret key'), max_length=64)
    default_currency = models.CharField(_('default currency'), max_length=3, choices=tuple(
        (currency, currency) for currency in CURRENCIES
    ))
    default_lang = models.CharField(_('default language'), max_length=5, choices=tuple(
        (lang, lang) for lang in LANGS
    ))

    def __str__(self):
        return self.name


class Payment(models.Model):
    NOT_REALIZED = 2
    REALIZED = 3
    STATES = {
        NOT_REALIZED: _('not realized'),
        REALIZED: _('successful'),
    }

    gateway = models.ForeignKey(Gateway, verbose_name=_('gateway'), on_delete=models.PROTECT, related_name='payments')
    created = models.DateTimeField(_('created'), auto_now_add=True)
    payment_id = models.CharField(_('payment identifier'), max_length=100)
    order_id = models.CharField(_('order identifier'), db_index=True, max_length=100)
    currency = models.CharField(_('currency'), max_length=3)
    amount = models.BigIntegerField(_('amount'), help_text=_('amount in the smallest (base) currency unit'))
    base_units = models.PositiveIntegerField(_('base units'), help_text=_('number of base units in currency'))
    status = models.PositiveSmallIntegerField(_('payment status'), choices=sorted(STATES.items()))
    status_description = models.TextField(_('payment status description'))

    class Meta:
        ordering = ('created',)
        unique_together = (('gateway', 'payment_id'),)
        verbose_name = _('payment')
        verbose_name_plural = _('payments')

    def __str__(self):
        return self.payment_id



__version__ = '1.21.3'
default_app_config = 'leprikon.apps.LeprikonConfig'

staticfiles_urls = {
    'leprikon/lib/bootstrap/css/bootstrap-theme.css': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/css/bootstrap-theme.css',  # NOQA
    'leprikon/lib/bootstrap/css/bootstrap.min.css': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/css/bootstrap.min.css',  # NOQA
    'leprikon/lib/bootstrap/css/bootstrap-theme.css.map': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/css/bootstrap-theme.css.map',  # NOQA
    'leprikon/lib/bootstrap/css/bootstrap.min.css.map': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/css/bootstrap.min.css.map',  # NOQA
    'leprikon/lib/bootstrap/css/bootstrap-theme.min.css': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/css/bootstrap-theme.min.css',  # NOQA
    'leprikon/lib/bootstrap/css/bootstrap.css': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/css/bootstrap.css',  # NOQA
    'leprikon/lib/bootstrap/css/bootstrap-theme.min.css.map': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/css/bootstrap-theme.min.css.map',  # NOQA
    'leprikon/lib/bootstrap/css/bootstrap.css.map': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/css/bootstrap.css.map',  # NOQA
    'leprikon/lib/bootstrap/fonts/glyphicons-halflings-regular.svg': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/fonts/glyphicons-halflings-regular.svg',  # NOQA
    'leprikon/lib/bootstrap/fonts/glyphicons-halflings-regular.ttf': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/fonts/glyphicons-halflings-regular.ttf',  # NOQA
    'leprikon/lib/bootstrap/fonts/glyphicons-halflings-regular.eot': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/fonts/glyphicons-halflings-regular.eot',  # NOQA
    'leprikon/lib/bootstrap/fonts/glyphicons-halflings-regular.woff': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/fonts/glyphicons-halflings-regular.woff',  # NOQA
    'leprikon/lib/bootstrap/fonts/glyphicons-halflings-regular.woff2': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/fonts/glyphicons-halflings-regular.woff2',  # NOQA
    'leprikon/lib/bootstrap/js/bootstrap.min.js': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/js/bootstrap.min.js',  # NOQA
    'leprikon/lib/bootstrap/js/npm.js': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/js/npm.js',  # NOQA
    'leprikon/lib/bootstrap/js/bootstrap.js': 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/js/bootstrap.js',  # NOQA
    'leprikon/lib/bootstrap-datetimepicker.min.css': ('https://cdnjs.cloudflare.com/ajax/libs/bootstrap-datetimepicker/4.17.37/css/bootstrap-datetimepicker.min.css', 'sha384', '/XmhRcB6e51B3NO1S5VVWnNKCDM74NlX0cmxZUwCJPg2P/0ECTkBKnjnzNnF7rX5'),  # NOQA
    'leprikon/lib/bootstrap-datetimepicker.min.js': ('https://cdnjs.cloudflare.com/ajax/libs/bootstrap-datetimepicker/4.17.37/js/bootstrap-datetimepicker.min.js', 'sha384', 'V936AaxnYTkq53cO2z8e42PFtCCdZhbRttfhgX8U1K9weNYVdCMbDetUKTtllFdc'),  # NOQA
    'leprikon/lib/bootstrap-multiselect.css': ('https://cdnjs.cloudflare.com/ajax/libs/bootstrap-multiselect/0.9.13/css/bootstrap-multiselect.css', 'sha384', 'l1tju+TyzPCuWXt+knuFYVhe2ZnnEm0/t9T8BmqNedtIBYj1fmsYrIAap4MU2amm'),  # NOQA
    'leprikon/lib/bootstrap-multiselect.min.js': ('https://cdnjs.cloudflare.com/ajax/libs/bootstrap-multiselect/0.9.13/js/bootstrap-multiselect.min.js', 'sha384', '2q+7Pk3OSOseTKfNi/t8mZzYRIovdg+nfXYeKmmnivkyD+nxosintmGKX7SWcrOG'),  # NOQA
    'leprikon/lib/ekko-lightbox.min.css': ('https://cdnjs.cloudflare.com/ajax/libs/ekko-lightbox/4.0.1/ekko-lightbox.min.css', 'sha384', 'EsoR0pjPTZTUy5b8kJf6aTfA7quYluD5mNQ/Lz4hsxU2kXuuc6PlXVfBJCFdTXqD'),  # NOQA
    'leprikon/lib/ekko-lightbox.min.js': ('https://cdnjs.cloudflare.com/ajax/libs/ekko-lightbox/4.0.1/ekko-lightbox.min.js', 'sha384', '2aVpK/uKiYlyFXHZ36wvxOPZyWeouEKvbSUlqCrkFhyT80n2aSYtT15Tgdn6g41M'),  # NOQA
    'leprikon/lib/jquery.min.js': ('https://cdnjs.cloudflare.com/ajax/libs/jquery/1.12.4/jquery.min.js', 'sha256', 'ZosEbRLbNQzLpnKIkEdrPv7lOy9C27hHQ+Xp8a4MxAQ='),  # NOQA
    'leprikon/lib/moment-with-locales.min.js': ('https://cdnjs.cloudflare.com/ajax/libs/moment.js/2.12.0/moment-with-locales.min.js', 'sha384', 'V0utPzNT28o7j37Ll4BIKqEVcL1yOE45HoCoaLu/CqF4nFaf2nAyZTHKJW1KdJMH'),  # NOQA
    'leprikon/lib/bootstrap-social.min.css': ('https://cdnjs.cloudflare.com/ajax/libs/bootstrap-social/5.1.1/bootstrap-social.min.css', 'sha384', 'N7gull5mxE2uLDqZ+kcZolhmuopwLtTcThUsVnUS2LVMQljojN9cfJ9A6fUA63zB'),  # NOQA
    'leprikon/lib/font-awesome/css/font-awesome.css': 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.css',  # NOQA 
    'leprikon/lib/font-awesome/css/font-awesome.css.map': 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.css.map',  # NOQA 
    'leprikon/lib/font-awesome/css/font-awesome.min.css': 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css',  # NOQA 
    'leprikon/lib/font-awesome/fonts/FontAwesome.otf': 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/fonts/FontAwesome.otf',  # NOQA 
    'leprikon/lib/font-awesome/fonts/fontawesome-webfont.eot': 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/fonts/fontawesome-webfont.eot',  # NOQA 
    'leprikon/lib/font-awesome/fonts/fontawesome-webfont.svg': 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/fonts/fontawesome-webfont.svg',  # NOQA 
    'leprikon/lib/font-awesome/fonts/fontawesome-webfont.ttf': 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/fonts/fontawesome-webfont.ttf',  # NOQA 
    'leprikon/lib/font-awesome/fonts/fontawesome-webfont.woff': 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/fonts/fontawesome-webfont.woff',  # NOQA 
    'leprikon/lib/font-awesome/fonts/fontawesome-webfont.woff2': 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/fonts/fontawesome-webfont.woff2',  # NOQA 
}



from django.db import models
from django.utils.translation import ugettext_lazy as _

from ..conf import settings


class UserAgreement(models.Model):
    user            = models.OneToOneField(settings.AUTH_USER_MODEL, primary_key=True, verbose_name=_('user'),
                                           related_name='agreement', on_delete=models.CASCADE)
    granted         = models.DateTimeField(_('time of agreement'), editable=False, auto_now=True)



from django.contrib import messages
from django.core.urlresolvers import reverse_lazy as reverse
from django.utils.http import is_safe_url
from django.utils.translation import ugettext_lazy as _
from django.views.generic import (
    CreateView as _CreateView, DeleteView as _DeleteView,
    DetailView as _DetailView, FormView as _FormView, ListView as _ListView,
    TemplateView as _TemplateView, UpdateView as _UpdateView,
)

from ..conf import settings
from ..forms.confirm import ConfirmForm


class GenericViewMixin(object):
    title       = ''

    def get_context_data(self, *args, **kwargs):
        context = super(GenericViewMixin, self).get_context_data(
            *args,
            placeholder = self.__class__.__name__,
            title       = self.get_title(),
            **kwargs
        )
        return context

    def get_title(self):
        return self.title



class ListView(GenericViewMixin, _ListView):
    template_name       = 'leprikon/list.html'
    add_url             = None
    add_label           = _('add')
    add_title           = ''
    message_empty       = _('No items found.')
    preview_template    = ''

    def get_add_url(self):
        return self.add_url

    def get_add_label(self):
        return self.add_label

    def get_add_title(self):
        return self.add_title

    def get_message_empty(self):
        return self.message_empty

    def get_context_data(self, *args, **kwargs):
        return super(ListView, self).get_context_data(
            *args,
            preview_template = self.preview_template,
            add_url = self.get_add_url(),
            add_label = self.get_add_label(),
            add_title = self.get_add_title(),
            message_empty = self.get_message_empty(),
            **kwargs
        )



class FilteredListView(ListView):
    form_class      = None
    message_empty   = _('No items found matching given search parameters.')

    def get_form(self):
        return self.form_class(data=self.request.GET)

    def get_context_data(self, *args, **kwargs):
        return super(FilteredListView, self).get_context_data(*args, form=self.get_form(), **kwargs)



class BackViewMixin(object):
    back_url    = reverse('leprikon:summary')
    back_label  = _('Back')

    def get_context_data(self, *args, **kwargs):
        return super(BackViewMixin, self).get_context_data(
            *args,
            back_url        = self.get_back_url(),
            back_label      = self.get_back_label(),
            **kwargs
        )

    def get_back_label(self):
        return self.back_label

    def get_back_url(self):
        url = self.request.POST.get(settings.LEPRIKON_PARAM_BACK,
                                    self.request.GET.get(settings.LEPRIKON_PARAM_BACK, ''))
        if is_safe_url(url=url, host=self.request.get_host()):
            return url
        else:
            return self.back_url



class FormViewMixin(BackViewMixin, GenericViewMixin):
    template_name   = 'leprikon/form.html'
    form_item_template_name = 'leprikon/form_item.html'
    instructions    = ''
    submit_label    = _('Save')
    success_url     = reverse('leprikon:summary')
    message         = None

    def get_context_data(self, *args, **kwargs):
        return super(FormViewMixin, self).get_context_data(
            *args,
            form_item_template = self.get_form_item_template_name(),
            instructions    = self.get_instructions(),
            submit_label    = self.get_submit_label(),
            **kwargs
        )

    def get_form_item_template_name(self):
        return self.form_item_template_name

    def get_instructions(self):
        return self.instructions

    def get_submit_label(self):
        return self.submit_label

    def get_success_url(self):
        return self.get_back_url()

    def get_message(self):
        return self.message

    def form_valid(self, form):
        response = super(FormViewMixin, self).form_valid(form)
        message = self.get_message()
        if message:
            messages.info(
                self.request,
                message,
            )
        return response



class GetFormViewMixin(FormViewMixin):
    def get_form_kwargs(self):
        return {
            'data': self.request.GET,
            'prefix': self.get_prefix(),
        }

    def get(self, reqest, *args, **kwargs):
        return self.post(self, reqest, *args, **kwargs)



class ConfirmFormViewMixin(FormViewMixin):
    form_class      = ConfirmForm
    template_name   = 'leprikon/confirm_form.html'
    question        = ''
    submit_label    = _('Yes')

    def get_question(self):
        return self.question

    def get_context_data(self, *args, **kwargs):
        return super(ConfirmFormViewMixin, self).get_context_data(*args, question=self.get_question(), **kwargs)

    def form_valid(self, form):
        self.confirmed()
        return super(ConfirmFormViewMixin, self).form_valid(form)

    def confirmed(self):
        pass



class DetailView(GenericViewMixin, _DetailView):
    pass



class CreateView(FormViewMixin, _CreateView):
    pass



class ConfirmCreateView(ConfirmFormViewMixin, _CreateView):
    pass



class UpdateView(FormViewMixin, _UpdateView):
    pass



class ConfirmUpdateView(ConfirmFormViewMixin, _UpdateView):
    pass



class DeleteView(ConfirmFormViewMixin, _DeleteView):
    submit_label = _('Delete')

    def delete(self, request, *args, **kwargs):
        response = super(DeleteView, self).delete(request, *args, **kwargs)
        message = self.get_message()
        if message:
            messages.info(
                self.request,
                message,
            )
        return response



class FormView(FormViewMixin, _FormView):
    pass



class GetFormView(GetFormViewMixin, _FormView):
    pass



class TemplateView(GenericViewMixin, _TemplateView):
    pass



from django import forms
from django.contrib import admin
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from ..forms.timesheets import TimesheetEntryAdminForm
from ..models.timesheets import TimesheetEntry
from .courses import CourseJournalLeaderEntryInlineAdmin
from .export import AdminExportMixin
from .filters import LeaderListFilter


class TimesheetEntryAdmin(AdminExportMixin, admin.ModelAdmin):
    form            = TimesheetEntryAdminForm
    date_hierarchy  = 'date'
    list_display    = ('timesheet', 'date', 'start', 'end', 'duration', 'entry_type')
    list_filter     = ('entry_type', ('timesheet__leader', LeaderListFilter))
    ordering        = ('-date', '-start',)

    def get_readonly_fields(self, request, obj=None):
        if obj and obj.timesheet.submitted:
            return ('timesheet', 'entry_type', 'date', 'start', 'end')
        return ()

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if obj and obj.timesheet.submitted:
            return False
        return super(TimesheetEntryAdmin, self).has_delete_permission(request, obj)



class TimesheetEntryInlineAdmin(admin.TabularInline):
    class form(forms.ModelForm):
        class Meta:
            model = TimesheetEntry
            fields = []
    model           = TimesheetEntry
    ordering        = ('date', 'start')
    readonly_fields = ('date', 'start', 'end', 'entry_type', 'description_html', 'edit_link')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if obj and obj.submitted:
            return False
        return super(TimesheetEntryInlineAdmin, self).has_delete_permission(request, obj)

    def description_html(self, obj):
        return obj.description
    description_html.short_description = _('description')
    description_html.admin_order_field = 'description'
    description_html.allow_tags = True

    def edit_link(self, obj):
        return '<a href="{url}" title="{title}" target="_blank">{edit}</a>'.format(
            url     = reverse('admin:leprikon_timesheetentry_change', args=[obj.id]),
            title   = _('update entry'),
            edit    = _('edit'),
        )
    edit_link.short_description = ''
    edit_link.allow_tags = True



class CJLEInlineAdmin(CourseJournalLeaderEntryInlineAdmin):
    readonly_fields = ('date', 'start', 'end', 'course', 'edit_link')



class TimesheetAdmin(AdminExportMixin, admin.ModelAdmin):
    list_display    = ('leader', 'period', 'group_durations', 'submitted', 'paid')
    list_filter     = (('leader', LeaderListFilter), 'period')
    inlines         = (TimesheetEntryInlineAdmin, CJLEInlineAdmin)
    actions         = ('submit', 'set_paid')

    # do not allow to add timesheets in admin
    # timesheets are created automatically
    def has_add_permission(self, request):
        return False

    # do not allow to add entries in admin (keep it simple)
    def has_delete_permission(self, request, obj=None):
        if obj:
            return not obj.submitted
        return False

    def get_actions(self, request):
        actions = super(TimesheetAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del(actions['delete_selected'])
        return actions

    def group_durations(self, obj):
        return '<br/>'.join(
            '<label>{name}</label>: {duration}'.format(
                name = group.name,
                duration = group.duration,
            ) for group in obj.groups
        )
    group_durations.allow_tags = True
    group_durations.short_description = _('group durations')

    def set_paid(self, request, queryset):
        queryset.update(submitted=True, paid=True)
        self.message_user(request, _('Selected timesheets where marked as paid.'))
    set_paid.short_description = _('Mark selected timesheets as paid')

    def submit(self, request, queryset):
        queryset.update(submitted=True)
        self.message_user(request, _('Selected timesheets where submitted.'))
    submit.short_description = _('Submit selected timesheets')



class TimesheetEntryTypeAdmin(admin.ModelAdmin):
    list_display    = ('name', 'order')
    list_editable   = ('order',)



from django.contrib import admin


class LeprikonSiteAdmin(admin.ModelAdmin):
    list_display    = ('name',)
    filter_horizontal = ('registration_agreements',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def get_actions(self, request):
        actions = super(LeprikonSiteAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del(actions['delete_selected'])
        return actions

    def get_fields(self, request, obj=None):
        fields = super(LeprikonSiteAdmin, self).get_fields(request, obj)
        if not obj or not obj.old_registration_agreement:
            fields.remove('old_registration_agreement')
        return fields



from django.conf.urls import url
from django.db import transaction

from .views import confirm

app_name = 'pays'

urlpatterns = [
    url(r'^(?P<slug>[^/]+)/', transaction.atomic(confirm), name='confirm'),
]
